import java.awt.BorderLayout;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.util.Dictionary;

import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Created on 02.07.2004 by Andreas Pleu
 * Rewritten 14.06.2005 by Heiko Drewes
 * Rewritten 26.06.2006 by Heiko Drewes and Richard Atterer
 * @author Andreas Pleu / Heiko Drewes / Richard Atterer
 */

/*
 * Do your programming in calcLens() at the end of this file
*/

public class ImageFrame extends JFrame
        implements MouseListener, MouseMotionListener, ComponentListener {

    private static final long serialVersionUID = 1L;

    // Sizes
    private int origWidth;
    private int origHeight;
    private int destWidth;
    private int destHeight;
    private int focusX;
    private int focusY;
    private int lensSize = 150;
    private double magnification = 3.0;

    // States
    private boolean showLens = false;
    private int fisheyeType = 0;
    // Images
    private BufferedImage origImg;
    private BufferedImage reducedImg;
    private BufferedImage lensImg = new BufferedImage(lensSize, lensSize, BufferedImage.TYPE_3BYTE_BGR);

    // Controls
    private JPanel controlPanel = new JPanel();
    private JSlider focusSlider;
    private JSlider magnificationSlider;
    private ImagePanel imagePanel;

    public static void main(String[] args) {

        // Get image filename from command line
        if (args.length < 1 || args[0].equals("")) {
            System.out.println("Specify filename of picture as command line parameter.");
            System.exit(1);
        }
        // Create and show window
        ImageFrame frame = new ImageFrame(args[0]);
        frame.setVisible(true);
    }

    public ImageFrame(String imgPath) {
        // Init UI
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setVisible(true);

        // Load image
        Image tmpImg = java.awt.Toolkit.getDefaultToolkit().createImage(imgPath);
        if (tmpImg == null)
            System.out.println("Error: could not load image");
        setIconImage(tmpImg);

        // Scale down image
        origWidth = tmpImg.getWidth(this);
        origHeight = tmpImg.getHeight(this);
        origImg =  new BufferedImage(origWidth, origHeight, BufferedImage.TYPE_3BYTE_BGR);

        // Trick to easily convert Image into a BufferedImage by
        // drawing the Image into the BufferedImage
        origImg.getGraphics().drawImage(tmpImg, 0, 0, this);
        setSize(800, origHeight * 800 / origWidth + 80);
        componentResized(null); // Create reducedImg, set destWidth/Height
        imagePanel = new ImagePanel();
        getContentPane().add(imagePanel, BorderLayout.CENTER);
        getContentPane().add(controlPanel, BorderLayout.SOUTH);
        initSliders();
        initRadioButtons();

        // For image only, listen to mouse clicks and drags
        imagePanel.addMouseListener(this);
        imagePanel.addMouseMotionListener(this);
        // Listen to window resize operations
        this.addComponentListener(this);
    }

    private void initSliders() {
        focusSlider = new JSlider(50, 400, 200);
        focusSlider.setValue(lensSize);
        focusSlider.setMinorTickSpacing(25);
        focusSlider.setMajorTickSpacing(100);
        focusSlider.setPaintTicks(true);
        focusSlider.setPaintLabels(true);
        focusSlider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();
                if (!source.getValueIsAdjusting())
                    repaint();
            }
        });
        Dictionary labels = focusSlider.getLabelTable();
        labels.put(new Integer(50), new JLabel("Size:"));
        focusSlider.setLabelTable(labels);
        controlPanel.add(focusSlider);

        magnificationSlider = new JSlider(50, 500, 300);
        magnificationSlider.setValue((int)(magnification * 100.0 + 0.5));
        magnificationSlider.setMinorTickSpacing(50);
        magnificationSlider.setMajorTickSpacing(100);
        magnificationSlider.setPaintTicks(true);
        magnificationSlider.setPaintLabels(true);
        magnificationSlider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();
                if (!source.getValueIsAdjusting())
                    repaint();
            }
        });
        labels = magnificationSlider.getLabelTable();
        labels.put(new Integer(500), new JLabel("%"));
        magnificationSlider.setLabelTable(labels);
        controlPanel.add(magnificationSlider);
    }

    private void initRadioButtons() {
        JRadioButton magButton = new JRadioButton("Magnify");
        JRadioButton fish1Button = new JRadioButton("Fisheye 1");
        JRadioButton fish2Button = new JRadioButton("Fisheye 2");
        magButton.setActionCommand("Magnify");
        fish1Button.setActionCommand("Fisheye1");
        fish2Button.setActionCommand("Fisheye2");
        magButton.setSelected(true);

        ActionListener buttonListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if(e.getActionCommand().equals("Fisheye1"))
                    fisheyeType = 1;
                else if(e.getActionCommand().equals("Fisheye2"))
                    fisheyeType = 2;
                else
                    fisheyeType = 0;
                repaint();
            }
        };

        magButton.addActionListener(buttonListener);
        fish1Button.addActionListener(buttonListener);
        fish2Button.addActionListener(buttonListener);
        ButtonGroup bGroup = new ButtonGroup();
        bGroup.add(magButton);
        controlPanel.add(magButton);
        bGroup.add(fish1Button);
        controlPanel.add(fish1Button);
        bGroup.add(fish2Button);
        controlPanel.add(fish2Button);
    }

    // Mouse events
    public void mousePressed(MouseEvent e) {
        Point p = e.getPoint();
        //System.out.println("Press: " + p.x + ", " + p.y);
        //System.out.println(e.paramString());

        if (e.getButton() == 1) {
            // Left mouse button
            focusX = p.x;
            focusY = p.y;
            showLens = true;
        }
        else {
            // Any other mouse button
            showLens = false;
        }
        repaint();
    }

    public void mouseClicked(MouseEvent e) { }

    public void mouseReleased(MouseEvent e) { }

    public void mouseEntered(MouseEvent e) { }

    public void mouseExited(MouseEvent e) { }

    public void mouseDragged(MouseEvent e) {
        Point p = e.getPoint();
        //System.out.println("Drag: " + p.x + ", " + p.y + ",   button " + e.getButton());
        //System.out.println(e.paramString());
        // Only repaint the bounding box of the old and new lens locations
        int rx0 = Math.min(focusX, p.x) - lensSize / 2 - 1;
        int ry0 = Math.min(focusY, p.y) - lensSize / 2 - 1;
        int rx1 = Math.max(focusX, p.x) + lensSize / 2 + 1;
        int ry1 = Math.max(focusY, p.y) + lensSize / 2 + 1;
        focusX = p.x;
        focusY = p.y;
        imagePanel.repaint(rx0, ry0, rx1 - rx0, ry1 - ry0);
    }

    public void mouseMoved(MouseEvent e) { }


    public void componentResized(ComponentEvent e) {
        destWidth = getWidth();
        destHeight = origHeight * destWidth / origWidth;
        reducedImg = new BufferedImage(destWidth, destHeight, BufferedImage.TYPE_3BYTE_BGR);
        reducedImg.getGraphics().drawImage(origImg, 0, 0, destWidth, destHeight, this);
    }

    public void componentMoved(ComponentEvent e) { }

    public void componentShown(ComponentEvent e) { }

    public void componentHidden(ComponentEvent e) {	}


    class ImagePanel extends JPanel {

        private static final long serialVersionUID = 1L;

        public void paint(Graphics g) {
            g.drawImage(reducedImg, 0, 0, this);
            if (showLens) {
                lensSize = focusSlider.getValue();
                magnification = magnificationSlider.getValue() / 100.0;
                lensImg = new BufferedImage(lensSize, lensSize, BufferedImage.TYPE_3BYTE_BGR);
                calcLens();
                g.drawImage(lensImg, focusX - lensSize / 2, focusY - lensSize / 2, this);
            }
        }

        // Do your programming here !!!
        // Add variables and complete the code for the case switch
        public void calcLens() {
            int rgbValue;
            int x, y;
            int w = lensImg.getWidth();
            int h = lensImg.getHeight();
            double origStretch = (double)origWidth / destWidth;
            int origWidth = origImg.getWidth();
            int origHeight = origImg.getHeight();

            switch (fisheyeType) {

            // Magnify
            case 0:
                for (int i = 0; i < w; ++i) {
                    for (int j = 0; j < h; ++j) {
                        x = (int)(origStretch * (focusX + (i - w / 2) / magnification) + 0.5);
                        y = (int)(origStretch * (focusY + (j - h / 2) / magnification) + 0.5);
                        // Better check whether still inside the image ;)
                        if (x >= 0 && y >= 0 && x < origWidth && y < origHeight)
                            rgbValue = origImg.getRGB(x, y);
                        else
                            rgbValue = 0;
                        lensImg.setRGB(i, j, rgbValue);
                    }
                }
                break;

            // Fisheye 1
            case 1:
                for (int i = 0; i < w; ++i) {
                    for (int j = 0; j < h; ++j) {
                        x = i;
                        y = j;
                        // Better check whether still inside the image ;)
                        if (x >= 0 && y >= 0 && x < origWidth && y < origHeight)
                            rgbValue = origImg.getRGB(x, y);
                        else
                            rgbValue = 0;
                        lensImg.setRGB(i, j, rgbValue);
                    }
                }
                break;

            // Fisheye 2
            case 2:
                for (int i = 0; i < w; ++i) {
                    for (int j = 0; j < h; ++j) {
                        x = i;
                        y = j;
                        // Better check whether still inside the image ;)
                        if (x >= 0 && y >= 0 && x < origWidth && y < origHeight)
                            rgbValue = origImg.getRGB(x, y);
                        else
                            rgbValue = 0;
                        lensImg.setRGB(i, j, rgbValue);
                    }
                }
                break;
            } // end switch
        }


    }

}
